@@ -83,6 +83,7 @@ define_keywords!(
     ADMIN,
     AFTER,
     AGAINST,
+    AGGREGATE,
     AGGREGATION,
     ALERT,
     ALGORITHM,
@@ -115,9 +116,11 @@ define_keywords!(
     AUTHENTICATION,
     AUTHORIZATION,
     AUTO,
+    AUTOEXTEND_SIZE,
     AUTOINCREMENT,
     AUTO_INCREMENT,
     AVG,
+    AVG_ROW_LENGTH,
     AVRO,
     BACKWARD,
     BASE64,
@@ -179,6 +182,7 @@ define_keywords!(
     CHARSET,
     CHAR_LENGTH,
     CHECK,
+    CHECKSUM,
     CIRCLE,
     CLEAR,
     CLOB,
@@ -268,11 +272,13 @@ define_keywords!(
     DEFINED,
     DEFINER,
     DELAYED,
+    DELAY_KEY_WRITE,
     DELETE,
     DELIMITED,
     DELIMITER,
     DELTA,
     DENSE_RANK,
+    DENY,
     DEREF,
     DESC,
     DESCRIBE,
@@ -287,6 +293,7 @@ define_keywords!(
     DISTRIBUTE,
     DIV,
     DO,
+    DOMAIN,
     DOUBLE,
     DOW,
     DOY,
@@ -311,6 +318,7 @@ define_keywords!(
     END_PARTITION,
     ENFORCED,
     ENGINE,
+    ENGINE_ATTRIBUTE,
     ENUM,
     ENUM16,
     ENUM8,
@@ -338,6 +346,7 @@ define_keywords!(
     EXPLAIN,
     EXPLICIT,
     EXPORT,
+    EXTEND,
     EXTENDED,
     EXTENSION,
     EXTERNAL,
@@ -441,6 +450,7 @@ define_keywords!(
     INPUTFORMAT,
     INSENSITIVE,
     INSERT,
+    INSERT_METHOD,
     INSTALL,
     INSTANT,
     INSTEAD,
@@ -477,6 +487,7 @@ define_keywords!(
     JULIAN,
     KEY,
     KEYS,
+    KEY_BLOCK_SIZE,
     KILL,
     LAG,
     LANGUAGE,
@@ -530,6 +541,7 @@ define_keywords!(
     MAX,
     MAXVALUE,
     MAX_DATA_EXTENSION_TIME_IN_DAYS,
+    MAX_ROWS,
     MEASURES,
     MEDIUMBLOB,
     MEDIUMINT,
@@ -551,6 +563,7 @@ define_keywords!(
     MINUTE,
     MINUTES,
     MINVALUE,
+    MIN_ROWS,
     MOD,
     MODE,
     MODIFIES,
@@ -648,6 +661,7 @@ define_keywords!(
     OWNERSHIP,
     PACKAGE,
     PACKAGES,
+    PACK_KEYS,
     PARALLEL,
     PARAMETER,
     PARQUET,
@@ -770,6 +784,7 @@ define_keywords!(
     ROW,
     ROWID,
     ROWS,
+    ROW_FORMAT,
     ROW_NUMBER,
     RULE,
     RUN,
@@ -784,6 +799,7 @@ define_keywords!(
     SEARCH,
     SECOND,
     SECONDARY,
+    SECONDARY_ENGINE_ATTRIBUTE,
     SECONDS,
     SECRET,
     SECURITY,
@@ -828,6 +844,7 @@ define_keywords!(
     SQLSTATE,
     SQLWARNING,
     SQRT,
+    SRID,
     STABLE,
     STAGE,
     START,
@@ -835,12 +852,16 @@ define_keywords!(
     STATEMENT,
     STATIC,
     STATISTICS,
+    STATS_AUTO_RECALC,
+    STATS_PERSISTENT,
+    STATS_SAMPLE_PAGES,
     STATUS,
     STDDEV_POP,
     STDDEV_SAMP,
     STDIN,
     STDOUT,
     STEP,
+    STORAGE,
     STORAGE_INTEGRATION,
     STORAGE_SERIALIZATION_POLICY,
     STORED,
@@ -867,6 +888,7 @@ define_keywords!(
     TABLE,
     TABLES,
     TABLESAMPLE,
+    TABLESPACE,
     TAG,
     TARGET,
     TASK,
@@ -982,6 +1004,7 @@ define_keywords!(
     WHEN,
     WHENEVER,
     WHERE,
+    WHILE,
     WIDTH_BUCKET,
     WINDOW,
     WITH,
@@ -1065,6 +1088,7 @@ pub const RESERVED_FOR_TABLE_ALIAS: &[Keyword] = &[
     Keyword::SAMPLE,
     Keyword::TABLESAMPLE,
     Keyword::FROM,
+    Keyword::OPEN,
 ];
 
 /// Can't be used as a column alias, so that `SELECT <expr> alias`
